#!/usr/bin/env python

"""
CourseBuilder 

Coursebuilder is a preprocessor tool for [pandoc](https://pandoc.org) 
to generate multi-lingual curricula documentation tables from 
structured representations as a flatfile database. Data scheme and 
actual values are kept in YAML files in order to version them with git.

"""

from argparse import ArgumentParser
import yaml
import string
import os,sys

from tablegenerator import TableGenerator
from markdowngenerator import MarkdownGenerator
from templategenerator import TemplateGenerator
from metagenerator import MetaGenerator


class CourseBuilder:

    @staticmethod
    def run():
        
        # arguments
        parser = ArgumentParser(description='versatile curricula generator')

        parser.add_argument('-m','--meta',action="extend", nargs="+", type=str,help="course description(s) as YAML file(s)")
        parser.add_argument('-l','--lang',help="Language to parse from meta file (use de or en)",default='de')
        parser.add_argument('-f','--fields',help="Fields to be used, the table will be build accordingly",action="extend", nargs="+", type=str)
        parser.add_argument('-s','--schema',help="using provided schema")
        parser.add_argument('-p','--pagebreak',action="store_true",help="add a pagebreak after each module")
        parser.add_argument('-t','--title',action="store_true",help="take first value in list as title")
        parser.add_argument('-b','--book',type=str,help="process a whole curriculum book with sections")
        parser.add_argument('--level',type=int,default=1,help="level of header tags")
        parser.add_argument('--table-gen',type=str,default=None,help='runs table generator')
        parser.add_argument('--template',type=str,default=None,help='defines a template to be used with fields')
        
        # get arguments
        args = parser.parse_args()
        
        if args.table_gen:

            tg = TableGenerator()
            
            tg.generate_table(args.table_gen)

            return


        # book mode with predefined setting from a book file
        if args.book and args.schema:
            
            generator = MetaGenerator()

            with open(args.schema) as sf:
                generator.set_schema(yaml.load(sf,Loader=yaml.Loader))

            with open(args.book) as bf:
                generator.process_book(yaml.load(bf,Loader=yaml.Loader),os.path.abspath(args.book),lang=args.lang,pagebreak=args.pagebreak,create_title=args.title,header_level=args.level)


        # verbose command line mode
        elif args.schema and args.meta and len(args.fields) > 0:
            
            generator = MetaGenerator()

            actual_fields = []

            if os.path.isfile(args.fields[0]):
                with open(args.fields[0]) as ff:
                    actual_fields = yaml.load(ff,Loader=yaml.Loader)['fields']
            else:
                actual_fields = args.fields 
            

            with open(args.schema) as f:
                generator.set_schema(yaml.load(f,Loader=yaml.Loader))

            for m in args.meta:
                with open(m) as fm:

                    table_items = generator.process(yaml.load(fm,Loader=yaml.Loader),fields=actual_fields,lang=args.lang,pagebreak=args.pagebreak,createTitle=args.title,header_level=args.level,template=args.template)

                    print(table_items)

        else:
            parser.print_help()

if __name__ == '__main__':
    CourseBuilder.run()
